@@ -7,13 +7,17 @@ import copy
 import datetime
 import os
 import re
-from typing import Dict, Iterator, List, Optional, Tuple
+from typing import Dict, Iterator, List, Optional
 
 from sarif import sarif_file_utils
+from sarif.sarif_file_utils import (
+    SARIF_SEVERITIES_WITHOUT_NONE,
+    SARIF_SEVERITIES_WITH_NONE,
+    read_result_severity,
+)
 from sarif.filter.general_filter import GeneralFilter
 from sarif.filter.filter_stats import FilterStats
-
-SARIF_SEVERITIES = ["error", "warning", "note"]
+from sarif.issues_report import IssuesReport
 
 BASIC_RECORD_ATTRIBUTES = [
     "Tool",
@@ -44,29 +48,6 @@ def has_sarif_file_extension(filename):
     return any(filename_upper.endswith(x) for x in [".SARIF", ".SARIF.JSON"])
 
 
-def _group_records_by_severity(records) -> Dict[str, List[Dict]]:
-    """
-    Get the records, grouped by severity.
-    """
-    return {
-        severity: [record for record in records if record["Severity"] == severity]
-        for severity in SARIF_SEVERITIES
-    }
-
-
-def _count_records_by_issue_code(records, severity) -> List[Tuple]:
-    """
-    Return a list of pairs (code_and_desc, count) of the records with the specified
-    severities.
-    """
-    code_to_count = {}
-    for record in records:
-        if record["Severity"] == severity:
-            code = combine_code_and_description(record)
-            code_to_count[code] = code_to_count.get(code, 0) + 1
-    return sorted(code_to_count.items(), key=lambda x: x[1], reverse=True)
-
-
 def get_record_headings(with_blame) -> List[str]:
     """
     Get the record headings for this SARIF file.
@@ -81,20 +62,6 @@ def get_record_headings(with_blame) -> List[str]:
     )
 
 
-def combine_code_and_description(record: dict) -> str:
-    """
-    Combine code and description fields into one string.
-    """
-    (code, description) = (record["Code"], record["Description"])
-    if code and description:
-        return f"{code.strip()} {description.strip()}"
-    if code:
-        return code.strip()
-    if description:
-        return description.strip()
-    return "<NONE>"
-
-
 def _add_filter_stats(accumulator, filter_stats):
     if filter_stats:
         if accumulator:
@@ -139,9 +106,11 @@ class SarifRun:
                     filter_date = conversion_driver["properties"].get("processed", None)
                     self._filter.rehydrate_filter_stats(
                         dehydrated_filter_stats,
-                        datetime.datetime.fromisoformat(filter_date)
-                        if filter_date
-                        else None,
+                        (
+                            datetime.datetime.fromisoformat(filter_date)
+                            if filter_date
+                            else None
+                        ),
                     )
 
     def init_path_prefix_stripping(self, autotrim=False, path_prefixes=None):
@@ -245,11 +214,12 @@ class SarifRun:
             ]
         return self._cached_records
 
-    def get_records_grouped_by_severity(self) -> Dict[str, List[Dict]]:
-        """
-        Get the records, grouped by severity.
-        """
-        return _group_records_by_severity(self.get_records())
+    def get_report(self) -> IssuesReport:
+        """Get the report, with records grouped and sorted for display."""
+        ret = IssuesReport()
+        for record in self.get_records():
+            ret.add_record(record)
+        return ret
 
     def result_to_record(self, result, include_blame_info=False):
         """
@@ -281,10 +251,7 @@ class SarifRun:
                         file_path = file_path[prefixlen:]
                     break
 
-        # Get the error severity, if included, and code
-        severity = result.get(
-            "level", "warning"
-        )  # If an error has no specified level then by default it is a warning
+        severity = read_result_severity(result, self.run_data)
 
         if "message" in result:
             # per RFC3629 At least one of the text (§3.11.8) or id (§3.11.10) properties SHALL be present https://docs.oasis-open.org/sarif/sarif/v2.1.0/os/sarif-v2.1.0-os.html#RFC3629
@@ -311,9 +278,11 @@ class SarifRun:
             "Line": line_number,
             "Severity": severity,
             "Code": error_id,
-            "Description": message[len(error_id) + 1].strip()
-            if message.startswith(error_id) and len(message) > len(error_id) + 1
-            else message,
+            "Description": (
+                message[len(error_id) + 1].strip()
+                if message.startswith(error_id) and len(message) > len(error_id) + 1
+                else message
+            ),
         }
         if include_blame_info:
             record["Author"] = _get_author_mail_from_blame_info(
@@ -328,23 +297,6 @@ class SarifRun:
         """
         return len(self.get_results())
 
-    def get_result_count_by_severity(self) -> Dict[str, int]:
-        """
-        Return a dict from SARIF severity to number of records.
-        """
-        records = self.get_records()
-        return {
-            severity: sum(1 for record in records if severity in record["Severity"])
-            for severity in SARIF_SEVERITIES
-        }
-
-    def get_issue_code_histogram(self, severity) -> List[Tuple]:
-        """
-        Return a list of pairs (code, count) of the records with the specified
-        severities.
-        """
-        return _count_records_by_issue_code(self.get_records(), severity)
-
     def get_filter_stats(self) -> Optional[FilterStats]:
         """
         Get the number of records that were included or excluded by the filter.
@@ -373,14 +325,36 @@ class SarifRun:
             else BASIC_RECORD_ATTRIBUTES
         )
 
+    def any_none(self) -> bool:
+        """Are there any records with severity "none"?"""
+        return any(
+            record.get("Severity", "warning") == "none" for record in self.get_records()
+        )
+
+    def get_severities(self, include_none=False) -> List[str]:
+        """
+        Get the severity list, with "none" omitted unless present or force-included.
+
+        Return ["error", "warning", "note", "none"] if include_none or if there
+        are any records with severity "none", otherwise ["error", "warning", "note"].
+        """
+        if include_none or self.any_none():
+            return SARIF_SEVERITIES_WITH_NONE
+        return SARIF_SEVERITIES_WITHOUT_NONE
+
 
 class SarifFile:
     """
     Class to hold SARIF data parsed from a file and provide accessors to the data.
     """
 
-    def __init__(self, file_path, data):
+    def __init__(self, file_path, data, mtime=None):
         self.abs_file_path = os.path.abspath(file_path)
+        if mtime:
+            self.mtime = mtime
+        else:
+            stat = os.stat(file_path)
+            self.mtime = datetime.datetime.fromtimestamp(stat.st_mtime)
         self.data = data
         self.runs = [
             SarifRun(self, run_index, run_data)
@@ -490,11 +464,12 @@ class SarifFile:
             ret += run.get_records()
         return ret
 
-    def get_records_grouped_by_severity(self) -> Dict[str, List[Dict]]:
-        """
-        Get the records, grouped by severity.
-        """
-        return _group_records_by_severity(self.get_records())
+    def get_report(self) -> IssuesReport:
+        """Get the report, with records grouped and sorted for display."""
+        ret = IssuesReport()
+        for record in self.get_records():
+            ret.add_record(record)
+        return ret
 
     def get_result_count(self) -> int:
         """
@@ -502,27 +477,6 @@ class SarifFile:
         """
         return sum(run.get_result_count() for run in self.runs)
 
-    def get_result_count_by_severity(self) -> Dict[str, int]:
-        """
-        Return a dict from SARIF severity to number of records.
-        """
-        result_count_by_severity_per_run = [
-            run.get_result_count_by_severity() for run in self.runs
-        ]
-        return {
-            severity: sum(
-                rc.get(severity, 0) for rc in result_count_by_severity_per_run
-            )
-            for severity in SARIF_SEVERITIES
-        }
-
-    def get_issue_code_histogram(self, severity) -> List[Tuple]:
-        """
-        Return a list of pairs (code, count) of the records with the specified
-        severities.
-        """
-        return _count_records_by_issue_code(self.get_records(), severity)
-
     def get_filter_stats(self) -> Optional[FilterStats]:
         """
         Get the number of records that were included or excluded by the filter.
@@ -536,6 +490,21 @@ class SarifFile:
         """Is there (any) blame info in this SARIF file?"""
         return any(run.has_blame_info() for run in self.runs)
 
+    def any_none(self) -> bool:
+        """Are there any records with severity "none"?"""
+        return any(run.any_none() for run in self.runs)
+
+    def get_severities(self, include_none=False) -> List[str]:
+        """
+        Get the severity list, with "none" omitted unless present or force-included.
+
+        Return ["error", "warning", "note", "none"] if include_none or if there
+        are any records with severity "none", otherwise ["error", "warning", "note"].
+        """
+        if include_none or self.any_none():
+            return SARIF_SEVERITIES_WITH_NONE
+        return SARIF_SEVERITIES_WITHOUT_NONE
+
 
 class SarifFileSet:
     """
@@ -566,10 +535,8 @@ class SarifFileSet:
         Iterate the SARIF files in this set.
         """
         for subdir in self.subdirs:
-            for input_file in subdir.files:
-                yield input_file
-        for input_file in self.files:
-            yield input_file
+            yield from subdir.files
+        yield from self.files
 
     def __getitem__(self, index) -> SarifFile:
         i = 0
@@ -683,12 +650,6 @@ class SarifFileSet:
             ret += input_file.get_records()
         return ret
 
-    def get_records_grouped_by_severity(self) -> Dict[str, List[Dict]]:
-        """
-        Get the records, grouped by severity.
-        """
-        return _group_records_by_severity(self.get_records())
-
     def get_result_count(self) -> int:
         """
         Return the total number of results.
@@ -697,26 +658,12 @@ class SarifFileSet:
             input_file.get_result_count() for input_file in self.files
         )
 
-    def get_result_count_by_severity(self) -> Dict[str, int]:
-        """
-        Return a dict from SARIF severity to number of records.
-        """
-        result_counts_by_severity = []
-        for subdir in self.subdirs:
-            result_counts_by_severity.append(subdir.get_result_count_by_severity())
-        for input_file in self.files:
-            result_counts_by_severity.append(input_file.get_result_count_by_severity())
-        return {
-            severity: sum(rc.get(severity, 0) for rc in result_counts_by_severity)
-            for severity in SARIF_SEVERITIES
-        }
-
-    def get_issue_code_histogram(self, severity) -> List[Tuple]:
-        """
-        Return a list of pairs (code, count) of the records with the specified
-        severities.
-        """
-        return _count_records_by_issue_code(self.get_records(), severity)
+    def get_report(self) -> IssuesReport:
+        """Get the report, with records grouped and sorted for display."""
+        ret = IssuesReport()
+        for record in self.get_records():
+            ret.add_record(record)
+        return ret
 
     def get_filter_stats(self) -> Optional[FilterStats]:
         """
@@ -732,3 +679,24 @@ class SarifFileSet:
     def has_blame_info(self) -> bool:
         """Is there (any) blame info in (any of) the SARIF files?"""
         return any(input_file.has_blame_info() for input_file in self.files)
+
+    def any_none(self) -> bool:
+        """Are there any records with severity "none"?"""
+        for subdir in self.subdirs:
+            if subdir.any_none():
+                return True
+        for input_file in self.files:
+            if input_file.any_none():
+                return True
+        return False
+
+    def get_severities(self, include_none=False) -> List[str]:
+        """
+        Get the severity list, with "none" omitted unless present or force-included.
+
+        Return ["error", "warning", "note", "none"] if include_none or if there
+        are any records with severity "none", otherwise ["error", "warning", "note"].
+        """
+        if include_none or self.any_none():
+            return SARIF_SEVERITIES_WITH_NONE
+        return SARIF_SEVERITIES_WITHOUT_NONE
